"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CharacterAnimator = CharacterAnimator;
exports.CharacterLoader = CharacterLoader;
exports.HanziWriter = HanziWriter;
exports.HanziWriterContext = void 0;
exports.HanziWriterGridLines = HanziWriterGridLines;
exports.QuizMistakeHighlighter = QuizMistakeHighlighter;
exports.UserStrokeGesture = UserStrokeGesture;
exports.defaultCharDataLoader = defaultCharDataLoader;
exports.useHanziWriter = void 0;
var _reactNative = require("react-native");
var _hanziWriter = require("./hanzi-writer");
var _withSelector = require("use-sync-external-store/with-selector");
var _react = require("react");
var _reactNativeSvg = _interopRequireWildcard(require("react-native-svg"));
var _LoadingIndicator = _interopRequireDefault(require("./components/LoadingIndicator"));
var _simplify = _interopRequireDefault(require("./simplify"));
var _geometryWorklet = require("./geometry-worklet");
var _StrokeAnimator = _interopRequireDefault(require("./components/StrokeAnimator"));
var _reactNativeReanimated = _interopRequireWildcard(require("react-native-reanimated"));
var _usePromise = require("./hooks/usePromise");
var _AnimatedPath = _interopRequireDefault(require("./components/AnimatedPath"));
var _reactNativeGestureHandler = require("react-native-gesture-handler");
var _utils = require("./utils");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
const HanziWriterContext = /*#__PURE__*/(0, _react.createContext)(null);
exports.HanziWriterContext = HanziWriterContext;
function HanziWriter(props) {
  return /*#__PURE__*/React.createElement(_reactNative.View, {
    style: [styles.hanziWriterRoot, props.style]
  }, /*#__PURE__*/React.createElement(HanziWriterContext.Provider, {
    value: props.writer
  }, /*#__PURE__*/React.createElement(CharacterLoader, {
    loading: props.loading,
    error: props.error
  }, props.children, /*#__PURE__*/React.createElement(UserStrokeGesture, null))));
}
HanziWriter.Svg = HanziWriterSvg;
HanziWriter.GridLines = HanziWriterGridLines;
HanziWriter.Outline = HanziWriterOutline;
HanziWriter.Character = HanziWriterCharacter;
HanziWriter.QuizMistakeHighlighter = QuizMistakeHighlighter;
HanziWriter.QuizStrokes = HanziWriterQuizStrokes;
async function defaultCharDataLoader(char) {
  return fetch(`https://cdn.jsdelivr.net/npm/hanzi-writer-data@2.0/${char}.json`).then(res => res.json());
}
function QuizMistakeHighlighter(_ref) {
  var _writer$characterClas;
  let {
    color = '#555',
    strokeDuration = 400
  } = _ref;
  const writer = (0, _react.useContext)(HanziWriterContext);
  const quiz = writer.quiz.useStore(s => s);
  const animationPath = (_writer$characterClas = writer.characterClass) === null || _writer$characterClas === void 0 ? void 0 : _writer$characterClas.strokes[quiz.index];
  const animationKey = (0, _react.useMemo)(() => {
    var _quiz$params, _quiz$params2, _quiz$params3;
    if (typeof ((_quiz$params = quiz.params) === null || _quiz$params === void 0 ? void 0 : _quiz$params.showHintAfterMisses) === 'boolean' && !((_quiz$params2 = quiz.params) !== null && _quiz$params2 !== void 0 && _quiz$params2.showHintAfterMisses)) {
      return null;
    }
    const showHintAfterMisses = ((_quiz$params3 = quiz.params) === null || _quiz$params3 === void 0 ? void 0 : _quiz$params3.showHintAfterMisses) || 3;
    const shouldAnimateStroke = (quiz.mistakes[quiz.index] || 0) >= showHintAfterMisses;
    if (shouldAnimateStroke) {
      return (0, _utils.generateId)();
    }
    return null;
  }, [quiz.mistakes, quiz.index, quiz.params]);
  if (!quiz.active || !animationPath || !animationKey) {
    return null;
  }
  return /*#__PURE__*/React.createElement(_reactNativeSvg.G, {
    transform: TRANSFORM
  }, /*#__PURE__*/React.createElement(_StrokeAnimator.default, {
    strokeColor: color,
    strokeWidth: 100,
    key: `mistake.${animationPath.strokeNum}.${animationKey}`,
    stroke: animationPath,
    resetOnComplete: true,
    duration: strokeDuration
  }));
}

/** Animates all strokes */
function CharacterAnimator(_ref2) {
  let {
    color = '#555',
    radicalColor
  } = _ref2;
  const writer = (0, _react.useContext)(HanziWriterContext);
  const animationState = writer.animator.useStore(s => s);
  if (animationState.state !== 'playing' || !writer.characterClass) {
    return null;
  }
  return /*#__PURE__*/React.createElement(React.Fragment, null, writer.characterClass.strokes.map((stroke, idx) => {
    const isLast = idx === writer.characterClass.strokes.length - 1;
    const colorToUse = stroke.isInRadical ? radicalColor : color;
    return /*#__PURE__*/React.createElement(_StrokeAnimator.default, {
      strokeColor: colorToUse || color,
      strokeWidth: 100,
      key: `char.${stroke.strokeNum}`,
      stroke: stroke,
      duration: animationState.strokeDuration,
      delay: idx * animationState.delayBetweenStrokes + animationState.strokeDuration * idx,
      onComplete: isLast ? writer.animator.onAnimationComplete : undefined
    });
  }));
}
function CharacterLoader(_ref3) {
  let {
    loading,
    error,
    children
  } = _ref3;
  const writer = (0, _react.useContext)(HanziWriterContext);
  switch (writer.characterState.status) {
    case 'idle':
      return null;
    case 'rejected':
      return error || /*#__PURE__*/React.createElement(_reactNative.Text, null, "Error loading character");
    case 'pending':
      return loading || /*#__PURE__*/React.createElement(_LoadingIndicator.default, null);
    default:
      return /*#__PURE__*/React.createElement(React.Fragment, null, children);
  }
}
const TRANSFORM = 'translate(20, 248.515625) scale(0.25390625, -0.25390625)';
const positioner = new _hanziWriter.Positioner({
  height: 300,
  width: 300,
  padding: 0
});

/** This component handles everything to do with the user's gestures on the writer element */
function UserStrokeGesture() {
  const writer = (0, _react.useContext)(HanziWriterContext);
  const points = (0, _reactNativeReanimated.useSharedValue)([]);
  const active = writer.quiz.useStore(state => state.active);
  const fade = (0, _reactNativeReanimated.useSharedValue)(1);
  const check = () => {
    const simplifiedPoints = (0, _simplify.default)(points.value, 1);
    writer.quiz.check(simplifiedPoints);
    // Fade out the user's stroke
    fade.value = (0, _reactNativeReanimated.withTiming)(0, {
      duration: 200
    }, () => {
      // reset the values
      points.value = [];
      fade.value = 1;
    });
  };
  const onGestureEvent = (0, _reactNativeReanimated.useAnimatedGestureHandler)({
    onStart(event) {
      points.value = [{
        x: event.x,
        y: event.y
      }];
    },
    onActive(event) {
      points.value = [...points.value, {
        x: event.x,
        y: event.y
      }];
    },
    onEnd() {
      if (points.value.length > 0) {
        (0, _reactNativeReanimated.runOnJS)(check)();
      }
    }
  });
  const animatedPathProps = (0, _reactNativeReanimated.useAnimatedProps)(() => ({
    d: (0, _geometryWorklet.getPathString)(points.value),
    opacity: fade.value
  }));
  return /*#__PURE__*/React.createElement(_reactNativeGestureHandler.PanGestureHandler, {
    enabled: active,
    onGestureEvent: onGestureEvent
  }, /*#__PURE__*/React.createElement(_reactNativeReanimated.default.View, {
    style: _reactNative.StyleSheet.absoluteFill
  }, /*#__PURE__*/React.createElement(_reactNativeSvg.default, {
    width: "300",
    height: "300"
  }, /*#__PURE__*/React.createElement(_reactNativeSvg.G, null, /*#__PURE__*/React.createElement(_AnimatedPath.default, {
    animatedProps: animatedPathProps,
    stroke: "black",
    strokeWidth: "2"
  })))));
}
function HanziWriterSvg(_ref4) {
  var _writer$characterClas2;
  let {
    children,
    ...rest
  } = _ref4;
  const writer = (0, _react.useContext)(HanziWriterContext);
  return /*#__PURE__*/React.createElement(_reactNativeSvg.default, _extends({
    width: "300",
    height: "300"
  }, rest), /*#__PURE__*/React.createElement(_reactNativeSvg.Defs, null, (_writer$characterClas2 = writer.characterClass) === null || _writer$characterClas2 === void 0 ? void 0 : _writer$characterClas2.strokes.map(stroke => /*#__PURE__*/React.createElement(_reactNativeSvg.ClipPath, {
    key: `clip.${stroke.strokeNum}`,
    id: `clip.${stroke.strokeNum}`
  }, /*#__PURE__*/React.createElement(_reactNativeSvg.Path, {
    d: stroke.path
  })))), children || null);
}
function HanziWriterQuizStrokes(props) {
  var _writer$characterClas3;
  const {
    color = '#555'
  } = props;
  const writer = (0, _react.useContext)(HanziWriterContext);
  const quizIndex = writer.quiz.useStore(state => state.index);
  const correctPaths = ((_writer$characterClas3 = writer.characterClass) === null || _writer$characterClas3 === void 0 ? void 0 : _writer$characterClas3.strokes.filter((_, i) => quizIndex > i)) || [];
  return /*#__PURE__*/React.createElement(_reactNativeSvg.G, {
    transform: TRANSFORM
  }, correctPaths.map(stroke => {
    const colorToUse = stroke.isInRadical ? props.radicalColor : color;
    return /*#__PURE__*/React.createElement(PathFadeIn, {
      key: `c.${stroke.strokeNum}`,
      d: stroke.path,
      fill: colorToUse || color
    });
  }));
}
function HanziWriterCharacter(props) {
  const writer = (0, _react.useContext)(HanziWriterContext);
  const {
    characterClass
  } = writer;
  const quizActive = writer.quiz.useStore(state => state.active);
  const isAnimating = writer.animator.useStore(state => state.state === 'playing');
  const color = props.color || '#555';
  const showCharStrokes = !quizActive && !isAnimating;
  return /*#__PURE__*/React.createElement(React.Fragment, null, showCharStrokes && /*#__PURE__*/React.createElement(_reactNativeSvg.G, {
    transform: TRANSFORM
  }, characterClass === null || characterClass === void 0 ? void 0 : characterClass.strokes.map(stroke => {
    const fillColor = stroke.isInRadical ? props.radicalColor : color;
    return /*#__PURE__*/React.createElement(_reactNativeSvg.Path, {
      key: `sc.${stroke.strokeNum}`,
      d: stroke.path,
      fill: fillColor || color
    });
  })), /*#__PURE__*/React.createElement(_reactNativeSvg.G, {
    transform: TRANSFORM
  }, /*#__PURE__*/React.createElement(CharacterAnimator, {
    color: color,
    radicalColor: props.radicalColor
  })));
}
function HanziWriterOutline(props) {
  var _writer$characterClas4;
  const writer = (0, _react.useContext)(HanziWriterContext);
  // const isQuizActive = writer?.quiz.useStore((s) => s.active);
  // const isAnimating = writer?.animator.useStore((s) => s.state === 'playing');

  const color = props.color || '#ededed';
  return /*#__PURE__*/React.createElement(_reactNativeSvg.G, {
    transform: TRANSFORM
  }, writer === null || writer === void 0 ? void 0 : (_writer$characterClas4 = writer.characterClass) === null || _writer$characterClas4 === void 0 ? void 0 : _writer$characterClas4.strokes.map(stroke => {
    return /*#__PURE__*/React.createElement(_reactNativeSvg.Path, {
      key: `o.${stroke.strokeNum}`,
      d: stroke.path,
      fill: color
    });
  }));
}
function PathFadeIn(props) {
  const opacity = (0, _reactNativeReanimated.useSharedValue)(0);
  (0, _react.useEffect)(() => {
    opacity.value = (0, _reactNativeReanimated.withTiming)(1, {
      duration: 300,
      easing: _reactNativeReanimated.Easing.linear
    });
  }, [opacity]);
  const animatedProps = (0, _reactNativeReanimated.useAnimatedProps)(() => {
    return {
      fillOpacity: opacity.value
    };
  });
  return /*#__PURE__*/React.createElement(_AnimatedPath.default, _extends({
    animatedProps: animatedProps
  }, props));
}
function HanziWriterGridLines(props) {
  const {
    color = '#eee',
    width = 3
  } = props;
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(_reactNative.View, {
    style: [styles.gridlineHorizontal, {
      borderColor: color,
      borderBottomWidth: width
    }]
  }), /*#__PURE__*/React.createElement(_reactNative.View, {
    style: [styles.gridlineVertical, {
      borderColor: color,
      borderLeftWidth: width
    }]
  }));
}
const styles = _reactNative.StyleSheet.create({
  hanziWriterRoot: {
    height: 300,
    width: 300,
    maxHeight: 300
  },
  gridlineHorizontal: {
    ..._reactNative.StyleSheet.absoluteFillObject,
    bottom: '50%',
    top: '50%'
  },
  gridlineVertical: {
    ..._reactNative.StyleSheet.absoluteFillObject,
    bottom: 0,
    left: '50%',
    top: 0,
    width: 1
  }
});
function createStoreApi(initialState) {
  let state = initialState;
  const subscribers = new Set();
  return {
    subscribe: listener => {
      subscribers.add(listener);
      return () => {
        subscribers.delete(listener);
      };
    },
    getState: () => state,
    setState(newState) {
      state = {
        ...state,
        ...newState
      };
      subscribers.forEach(listener => listener());
    }
  };
}
const useCharacterAnimator = params => {
  const animationKeyRef = (0, _react.useRef)('');
  const characterAnimatorStore = (0, _react.useMemo)(() => {
    return createStoreApi({
      character: params.character,
      state: 'stopped',
      animationKey: null,
      onComplete: null,
      delayBetweenStrokes: 1500,
      strokeDuration: 400
    });
  }, [params.character]);
  return {
    useStore(selector, equalityFn) {
      return useStore(characterAnimatorStore, selector, equalityFn);
    },
    store: characterAnimatorStore,
    animateCharacter(params) {
      const {
        strokeDuration = 400,
        delayBetweenStrokes = 1500
      } = params || {};
      animationKeyRef.current = (0, _utils.generateId)();
      characterAnimatorStore.setState({
        state: 'playing',
        animationKey: animationKeyRef.current,
        onComplete: params === null || params === void 0 ? void 0 : params.onComplete,
        delayBetweenStrokes,
        strokeDuration
      });
    },
    cancelAnimation() {
      characterAnimatorStore.setState({
        state: 'stopped'
      });
    },
    onAnimationComplete: () => {
      const {
        animationKey
      } = characterAnimatorStore.getState();
      // Avoid race conditions
      if (animationKeyRef.current === animationKey) {
        var _characterAnimatorSto, _characterAnimatorSto2;
        (_characterAnimatorSto = (_characterAnimatorSto2 = characterAnimatorStore.getState()).onComplete) === null || _characterAnimatorSto === void 0 ? void 0 : _characterAnimatorSto.call(_characterAnimatorSto2);
        characterAnimatorStore.setState({
          state: 'stopped'
        });
      }
    }
  };
};
const createQuizStore = character => {
  const api = createStoreApi({
    character,
    active: false,
    mistakes: {},
    index: 0,
    params: null
  });
  return {
    ...api,
    next() {
      api.setState({
        index: api.getState().index + 1
      });
    }
  };
};
function useStore(api, selector, equalityFn) {
  return (0, _withSelector.useSyncExternalStoreWithSelector)(api.subscribe, api.getState, api.getState, selector, equalityFn);
}
const useQuiz = params => {
  const {
    cancelAnimation,
    character,
    characterClass
  } = params;
  const quizStore = (0, _react.useMemo)(() => createQuizStore(character), [character]);
  const check = (0, _react.useCallback)(simplifiedPoints => {
    if (!characterClass) {
      return;
    }
    const {
      index,
      mistakes,
      params
    } = quizStore.getState();
    const [firstExternalPoint, ...restExternalPoints] = simplifiedPoints;
    const firstPoint = positioner.convertExternalPoint(firstExternalPoint);
    const userStroke = new _hanziWriter.UserStroke(index, firstPoint, firstExternalPoint);
    for (const externalPoint of restExternalPoints) {
      const offset = positioner.convertExternalPoint(externalPoint);
      userStroke.appendPoint(offset, externalPoint);
    }
    const {
      isMatch,
      meta
    } = (0, _hanziWriter.strokeMatches)(userStroke, characterClass, index, {
      leniency: (params === null || params === void 0 ? void 0 : params.leniency) || 1.2
    });
    const isAccepted = isMatch || meta.isStrokeBackwards && (params === null || params === void 0 ? void 0 : params.acceptBackwardsStrokes);
    if (!isAccepted) {
      var _quizStore$getState$p, _quizStore$getState$p2;
      const numMistakes = (mistakes[index] || 0) + 1;
      quizStore.setState({
        mistakes: {
          ...mistakes,
          [index]: numMistakes
        }
      });
      (_quizStore$getState$p = quizStore.getState().params) === null || _quizStore$getState$p === void 0 ? void 0 : (_quizStore$getState$p2 = _quizStore$getState$p.onMistake) === null || _quizStore$getState$p2 === void 0 ? void 0 : _quizStore$getState$p2.call(_quizStore$getState$p, {
        character,
        drawnPath: {
          pathString: '',
          points: userStroke.points
        },
        isBackwards: meta.isStrokeBackwards,
        mistakesOnStroke: numMistakes,
        strokeNum: index,
        strokesRemaining: characterClass.strokes.length - index,
        totalMistakes: Object.keys(mistakes).length
      });
    } else {
      const active = index < characterClass.strokes.length - 1;
      if (!active) {
        var _params$onComplete;
        params === null || params === void 0 ? void 0 : (_params$onComplete = params.onComplete) === null || _params$onComplete === void 0 ? void 0 : _params$onComplete.call(params, {
          totalMistakes: Object.keys(mistakes).length,
          character: characterClass.symbol
        });
      }
      quizStore.setState({
        index: active ? index + 1 : 0,
        active
      });
    }
  }, [character, characterClass, quizStore]);
  return {
    store: quizStore,
    useStore(selector, equalityFn) {
      return useStore(quizStore, selector, equalityFn);
    },
    check,
    start: params => {
      if (!characterClass) {
        console.warn("Can't start quiz, character not loaded yet");
        return;
      }
      // Cancel any running animations
      cancelAnimation();

      // Start from an index between 0 and the number of strokes
      const index = Math.max(Math.min(params.quizStartStrokeNum || 0, characterClass.strokes.length - 1), 0);
      quizStore.setState({
        active: true,
        index,
        mistakes: {},
        params
      });
    },
    stop: () => {
      quizStore.setState({
        active: false,
        index: 0,
        mistakes: {},
        params: null
      });
    }
  };
};
const useHanziWriter = params => {
  const {
    character
  } = params;
  const promise = (0, _react.useCallback)(async () => {
    const loader = params.loader || defaultCharDataLoader;
    const value = loader(character);
    if (value instanceof Promise) {
      return value.then(res => (0, _hanziWriter.parseCharData)(character, res));
    }
    return (0, _hanziWriter.parseCharData)(character, value);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [character]);
  const {
    state: characterState,
    refetch
  } = (0, _usePromise.usePromise)({
    cacheKey: `char.${character}`,
    promiseFn: promise
  });
  const animator = useCharacterAnimator({
    character
  });
  const characterClass = characterState.status === 'resolved' ? characterState.data : null;
  const quiz = useQuiz({
    character,
    characterClass,
    cancelAnimation: animator.cancelAnimation
  });
  return {
    animator,
    quiz,
    characterState,
    characterClass,
    refetch
  };
};
exports.useHanziWriter = useHanziWriter;
//# sourceMappingURL=HanziWriter.js.map