"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = StrokeAnimator;
var _react = require("react");
var _reactNativeReanimated = require("react-native-reanimated");
var _AnimatedPath = _interopRequireDefault(require("../components/AnimatedPath"));
var _geometry = require("../geometry");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function StrokeAnimator(_ref) {
  let {
    stroke,
    duration,
    delay,
    strokeColor,
    strokeWidth,
    resetOnComplete,
    onComplete
  } = _ref;
  const progress = (0, _reactNativeReanimated.useSharedValue)(0);
  (0, _react.useEffect)(() => {
    const onCompleteCallback = () => onComplete === null || onComplete === void 0 ? void 0 : onComplete();
    const strokeIn = (0, _reactNativeReanimated.withTiming)(1, {
      duration,
      easing: _reactNativeReanimated.Easing.linear
    }, () => (0, _reactNativeReanimated.runOnJS)(onCompleteCallback)());
    const animations = [delay ? (0, _reactNativeReanimated.withDelay)(delay, strokeIn) : strokeIn];
    if (resetOnComplete) {
      animations.push((0, _reactNativeReanimated.withDelay)(150, (0, _reactNativeReanimated.withTiming)(0, {
        duration: 0,
        easing: _reactNativeReanimated.Easing.ease
      })));
    }
    if (animations.length < 2) {
      progress.value = animations[0];
    } else {
      // @ts-ignore
      progress.value = (0, _reactNativeReanimated.withSequence)(...animations);
    }
  }, [resetOnComplete, onComplete, progress, delay, duration]);
  const path = (0, _react.useMemo)(() => {
    const extendedMaskPoints = (0, _geometry.extendStart)(stroke.points, strokeWidth / 2);
    return (0, _geometry.getPathString)(extendedMaskPoints);
  }, [stroke.points, strokeWidth]);
  const pathLength = stroke.getLength() + strokeWidth / 2;
  return /*#__PURE__*/React.createElement(AnimatedStroke, {
    clipPath: `url(#clip.${stroke.strokeNum})`,
    progress: progress,
    d: path,
    color: strokeColor,
    pathLength: pathLength
  });
}
function AnimatedStroke(_ref2) {
  let {
    d,
    progress,
    clipPath,
    color,
    pathLength
  } = _ref2;
  const ref = (0, _react.useRef)(null);
  const animatedProps = (0, _reactNativeReanimated.useAnimatedProps)(() => {
    return {
      strokeDashoffset: pathLength * (1 - progress.value),
      // Ease out the stroke opacity
      strokeOpacity: progress.value < 0.01 ? progress.value * 2 : 1
    };
  });
  return /*#__PURE__*/React.createElement(_AnimatedPath.default, {
    ref: ref,
    animatedProps: animatedProps,
    d: d,
    stroke: color,
    clipPath: clipPath,
    strokeWidth: 180,
    strokeLinecap: "round",
    fill: "none",
    strokeDasharray: `${pathLength},${pathLength}`
  });
}
//# sourceMappingURL=StrokeAnimator.js.map