"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.usePromise = usePromise;
var _react = require("react");
const PromiseCache = new Map();
/** Simple implementation of a React hook for handling promises with caching. */
function usePromise(options) {
  const [state, setState] = (0, _react.useState)({
    status: 'idle'
  });
  const [refetchKey, setRefetchKey] = (0, _react.useState)(0);
  const hasCacheHit = PromiseCache.get(options.cacheKey || '');
  (0, _react.useEffect)(() => {
    if (!options.promiseFn || hasCacheHit) {
      return;
    }
    let safeSetState = setState;
    safeSetState({
      status: 'pending'
    });
    options.promiseFn().then(data => {
      safeSetState({
        status: 'resolved',
        data
      });
      if (options.cacheKey) {
        PromiseCache.set(options.cacheKey, {
          status: 'resolved',
          data
        });
      }
    }).catch(error => {
      safeSetState({
        status: 'rejected',
        error
      });
    });
    return () => {
      safeSetState = () => {};
    };
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [options.cacheKey, hasCacheHit, refetchKey]);
  const characterState = hasCacheHit ? PromiseCache.get(options.cacheKey || '') : state;
  return {
    state: characterState,
    refetch: () => setRefetchKey(refetchKey + 1)
  };
}
//# sourceMappingURL=usePromise.js.map