import { useEffect, useMemo, useRef } from 'react';
import { Easing, runOnJS, useAnimatedProps, useSharedValue, withDelay, withSequence, withTiming } from 'react-native-reanimated';
import AnimatedPath from '../components/AnimatedPath';
import { extendStart, getPathString } from '../geometry';
export default function StrokeAnimator(_ref) {
  let {
    stroke,
    duration,
    delay,
    strokeColor,
    strokeWidth,
    resetOnComplete,
    onComplete
  } = _ref;
  const progress = useSharedValue(0);
  useEffect(() => {
    const onCompleteCallback = () => onComplete === null || onComplete === void 0 ? void 0 : onComplete();
    const strokeIn = withTiming(1, {
      duration,
      easing: Easing.linear
    }, () => runOnJS(onCompleteCallback)());
    const animations = [delay ? withDelay(delay, strokeIn) : strokeIn];
    if (resetOnComplete) {
      animations.push(withDelay(150, withTiming(0, {
        duration: 0,
        easing: Easing.ease
      })));
    }
    if (animations.length < 2) {
      progress.value = animations[0];
    } else {
      // @ts-ignore
      progress.value = withSequence(...animations);
    }
  }, [resetOnComplete, onComplete, progress, delay, duration]);
  const path = useMemo(() => {
    const extendedMaskPoints = extendStart(stroke.points, strokeWidth / 2);
    return getPathString(extendedMaskPoints);
  }, [stroke.points, strokeWidth]);
  const pathLength = stroke.getLength() + strokeWidth / 2;
  return /*#__PURE__*/React.createElement(AnimatedStroke, {
    clipPath: `url(#clip.${stroke.strokeNum})`,
    progress: progress,
    d: path,
    color: strokeColor,
    pathLength: pathLength
  });
}
function AnimatedStroke(_ref2) {
  let {
    d,
    progress,
    clipPath,
    color,
    pathLength
  } = _ref2;
  const ref = useRef(null);
  const animatedProps = useAnimatedProps(() => {
    return {
      strokeDashoffset: pathLength * (1 - progress.value),
      // Ease out the stroke opacity
      strokeOpacity: progress.value < 0.01 ? progress.value * 2 : 1
    };
  });
  return /*#__PURE__*/React.createElement(AnimatedPath, {
    ref: ref,
    animatedProps: animatedProps,
    d: d,
    stroke: color,
    clipPath: clipPath,
    strokeWidth: 180,
    strokeLinecap: "round",
    fill: "none",
    strokeDasharray: `${pathLength},${pathLength}`
  });
}
//# sourceMappingURL=StrokeAnimator.js.map