import { StyleProp, ViewStyle } from 'react-native';
import { Character } from './hanzi-writer';
import type { Point, QuizOptions, CharacterJson } from './types';
import { ReactNode } from 'react';
import { SvgProps } from 'react-native-svg';
export declare const HanziWriterContext: import("react").Context<{
    animator: {
        useStore<T>(selector: (state: CharacterAnimatorState) => T, equalityFn?: any): T;
        store: {
            subscribe: (listener: () => void) => () => void;
            getState: () => CharacterAnimatorState;
            setState(newState: Partial<CharacterAnimatorState>): void;
        };
        animateCharacter(params?: {
            /** In milliseconds. Default: 400 */
            strokeDuration?: number;
            /** In milliseconds. Default: 1500 */
            delayBetweenStrokes?: number;
            onComplete?: () => void;
        }): void;
        cancelAnimation(): void;
        onAnimationComplete: () => void;
    };
    quiz: {
        store: {
            next(): void;
            subscribe: (listener: () => void) => () => void;
            getState: () => QuizState;
            setState(newState: Partial<QuizState>): void;
        };
        useStore<T_1>(selector: (state: QuizState) => T_1, equalityFn?: any): T_1;
        check: (simplifiedPoints: Point[]) => void;
        start: (params: StartQuizParams) => void;
        stop: () => void;
    };
    characterState: {
        status: "idle";
    } | {
        status: "pending";
    } | {
        status: "rejected";
        error: Error;
    } | {
        status: "resolved";
        data: Character;
    };
    characterClass: Character | null;
    refetch: () => void;
} | null>;
interface HanziWriterProps {
    writer: ReturnType<typeof useHanziWriter>;
    children: ReactNode;
    /** What to display while the character is loading */
    loading?: JSX.Element;
    /** What to display if there's an error */
    error?: JSX.Element;
    /** Container style */
    style?: StyleProp<ViewStyle>;
}
export declare function HanziWriter(props: HanziWriterProps): JSX.Element;
export declare namespace HanziWriter {
    var Svg: typeof HanziWriterSvg;
    var GridLines: typeof HanziWriterGridLines;
    var Outline: typeof HanziWriterOutline;
    var Character: typeof HanziWriterCharacter;
    var QuizMistakeHighlighter: typeof import("./HanziWriter").QuizMistakeHighlighter;
    var QuizStrokes: typeof HanziWriterQuizStrokes;
}
export declare function defaultCharDataLoader(char: string): Promise<CharacterJson>;
export declare function QuizMistakeHighlighter({ color, strokeDuration, }: {
    color?: string | undefined;
    strokeDuration?: number | undefined;
}): JSX.Element | null;
/** Animates all strokes */
export declare function CharacterAnimator({ color, radicalColor, }: {
    color?: string;
    radicalColor?: string;
}): JSX.Element | null;
interface CharacterLoaderProps {
    /** What to display while the character is loading */
    loading?: JSX.Element;
    /** What to display if there's an error */
    error?: JSX.Element;
    /** Resolved character data */
    children: ReactNode;
}
export declare function CharacterLoader({ loading, error, children, }: CharacterLoaderProps): JSX.Element | null;
export type HanziWriterAnimationState = 'playing' | 'paused' | 'stopped' | 'cancelled';
/** This component handles everything to do with the user's gestures on the writer element */
export declare function UserStrokeGesture(): JSX.Element;
interface HanziWriterSvgProps extends SvgProps {
    children?: ReactNode;
}
declare function HanziWriterSvg({ children, ...rest }: HanziWriterSvgProps): JSX.Element;
declare function HanziWriterQuizStrokes(props: {
    color?: string;
    radicalColor?: string;
}): JSX.Element;
declare function HanziWriterCharacter(props: {
    color?: string;
    radicalColor?: string;
}): JSX.Element;
declare function HanziWriterOutline(props: {
    color?: string;
}): JSX.Element;
export declare function HanziWriterGridLines(props: {
    color?: string;
    width?: number;
}): JSX.Element;
type AvailableQuizOptions = Omit<QuizOptions, 'highlightOnComplete'>;
type StartQuizParams = Partial<AvailableQuizOptions>;
interface CharacterAnimatorState {
    character: string;
    state: 'playing' | 'stopped';
    animationKey: string | null;
    onComplete: (() => void) | null;
    delayBetweenStrokes: number;
    strokeDuration: number;
}
type QuizState = {
    readonly active: false;
    character: string;
    index: number;
    mistakes: Record<number, number>;
    params: null;
} | {
    readonly active: true;
    character: string;
    index: number;
    mistakes: Record<number, number>;
    params: StartQuizParams;
};
export declare const useHanziWriter: (params: {
    character: string;
    loader?: ((char: string) => Promise<CharacterJson> | CharacterJson) | undefined;
}) => {
    animator: {
        useStore<T>(selector: (state: CharacterAnimatorState) => T, equalityFn?: any): T;
        store: {
            subscribe: (listener: () => void) => () => void;
            getState: () => CharacterAnimatorState;
            setState(newState: Partial<CharacterAnimatorState>): void;
        };
        animateCharacter(params?: {
            /** In milliseconds. Default: 400 */
            strokeDuration?: number;
            /** In milliseconds. Default: 1500 */
            delayBetweenStrokes?: number;
            onComplete?: () => void;
        }): void;
        cancelAnimation(): void;
        onAnimationComplete: () => void;
    };
    quiz: {
        store: {
            next(): void;
            subscribe: (listener: () => void) => () => void;
            getState: () => QuizState;
            setState(newState: Partial<QuizState>): void;
        };
        useStore<T_1>(selector: (state: QuizState) => T_1, equalityFn?: any): T_1;
        check: (simplifiedPoints: Point[]) => void;
        start: (params: StartQuizParams) => void;
        stop: () => void;
    };
    characterState: {
        status: "idle";
    } | {
        status: "pending";
    } | {
        status: "rejected";
        error: Error;
    } | {
        status: "resolved";
        data: Character;
    };
    characterClass: Character | null;
    refetch: () => void;
};
export {};
//# sourceMappingURL=HanziWriter.d.ts.map